@@ -1,5 +1,6 @@
 #include <R.h>
 #include <Rinternals.h>
+#include <R_ext/RS.h>
 #include <stdlib.h> // for NULL
 #include <R_ext/Rdynload.h>
 
@@ -14,6 +15,9 @@ extern SEXP test_cox_tvc2_grad(SEXP);
 extern SEXP fitCureModel(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
 extern SEXP aft_model_output(SEXP);
 
+/* .Fortran calls -- thanks for Gordon Smyth */
+extern void F77_NAME(gausq2)(void *, void *, void *, void *, void *);
+
 static const R_CallMethodDef CallEntries[] = {
     {"model_output",       (DL_FUNC) &model_output,       1},
     {"test_cox_tvc2",      (DL_FUNC) &test_cox_tvc2,      1},
@@ -23,8 +27,14 @@ static const R_CallMethodDef CallEntries[] = {
     {NULL, NULL, 0}
 };
 
+static const R_FortranMethodDef FortranEntries[] = {
+    {"gausq2", (DL_FUNC) &F77_NAME(gausq2), 5},
+    {NULL, NULL, 0}
+};
+
 void R_init_rstpm2(DllInfo *dll)
 {
     R_registerRoutines(dll, NULL, CallEntries, NULL, NULL);
+    R_registerRoutines(dll, NULL, NULL, FortranEntries, NULL);
     R_useDynamicSymbols(dll, FALSE);
 }
